@@ -1,6 +1,8 @@
 # python-musicpd: Python MPD client library
 # Copyright (C) 2008-2010  J. Alexander Treuman <jat@spatialrift.net>
-# Copyright (C) 2012-2018  Kaliko Jack <kaliko@azylum.org>
+# Copyright (C) 2012-2019  Kaliko Jack <kaliko@azylum.org>
+# Copyright (C) 2019       Naglis Jonaitis <naglis@mailbox.org>
+# Copyright (C) 2019       Bart Van Loon <bbb@bbbart.be>
 #
 # python-musicpd is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -27,7 +29,8 @@ HELLO_PREFIX = "OK MPD "
 ERROR_PREFIX = "ACK "
 SUCCESS = "OK"
 NEXT = "list_OK"
-VERSION = '0.4.3'
+VERSION = '0.4.4'
+CONNECTION_TIMEOUT = 5  # seconds before a tcp connection attempt times out
 
 
 def iterator_wrapper(func):
@@ -38,6 +41,7 @@ def iterator_wrapper(func):
         if not instance.iterate:
             return list(generator)
         instance._iterating = True
+
         def iterator(gen):
             try:
                 for item in gen:
@@ -51,24 +55,31 @@ def iterator_wrapper(func):
 class MPDError(Exception):
     pass
 
+
 class ConnectionError(MPDError):
     pass
 
+
 class ProtocolError(MPDError):
     pass
 
+
 class CommandError(MPDError):
     pass
 
+
 class CommandListError(MPDError):
     pass
 
+
 class PendingCommandError(MPDError):
     pass
 
+
 class IteratingError(MPDError):
     pass
 
+
 class Range:
 
     def __init__(self, tpl):
@@ -96,14 +107,18 @@ class Range:
             except (TypeError, ValueError):
                 raise CommandError('Not a tuple of int')
 
+
 class _NotConnected:
+
     def __getattr__(self, attr):
         return self._dummy
 
     def _dummy(*args):
         raise ConnectionError("Not connected")
 
+
 class MPDClient:
+
     """MPDClient instance will look for ``MPD_HOST``/``MPD_PORT``/``XDG_RUNTIME_DIR`` environment
     variables and set instance attribute ``host``, ``port`` and ``pwd``
     accordingly.
@@ -112,7 +127,7 @@ class MPDClient:
 
     Cf. :py:obj:`musicpd.MPDClient.connect` for details.
 
-    >>> from os inport environ
+    >>> from os import environ
     >>> environ['MPD_HOST'] = 'pass@mpdhost'
     >>> cli = musicpd.MPDClient()
     >>> cli.pwd == environ['MPD_HOST'].split('@')[0]
@@ -155,7 +170,7 @@ class MPDClient:
             "seekid":             self._fetch_nothing,
             "seekcur":            self._fetch_nothing,
             "stop":               self._fetch_nothing,
-            # Playlist Commands
+            # Queue Commands
             "add":                self._fetch_nothing,
             "addid":              self._fetch_item,
             "clear":              self._fetch_nothing,
@@ -191,19 +206,21 @@ class MPDClient:
             "rm":                 self._fetch_nothing,
             "save":               self._fetch_nothing,
             # Database Commands
+            "albumart":           self._fetch_composite,
             "count":              self._fetch_object,
             "find":               self._fetch_songs,
             "findadd":            self._fetch_nothing,
             "list":               self._fetch_list,
             "listall":            self._fetch_database,
             "listallinfo":        self._fetch_database,
+            "listfiles":          self._fetch_database,
             "lsinfo":             self._fetch_database,
+            "readcomments":       self._fetch_object,
             "search":             self._fetch_songs,
             "searchadd":          self._fetch_nothing,
             "searchaddpl":        self._fetch_nothing,
             "update":             self._fetch_item,
             "rescan":             self._fetch_item,
-            "readcomments":       self._fetch_object,
             # Mounts and neighbors
             "mount":              self._fetch_nothing,
             "unmount":            self._fetch_nothing,
@@ -220,16 +237,25 @@ class MPDClient:
             "kill":               None,
             "password":           self._fetch_nothing,
             "ping":               self._fetch_nothing,
+            "tagtypes":           self._fetch_list,
+            "tagtypes disable":   self._fetch_nothing,
+            "tagtypes enable":    self._fetch_nothing,
+            "tagtypes clear":     self._fetch_nothing,
+            "tagtypes all":       self._fetch_nothing,
+            # Partition Commands
+            "partition":          self._fetch_nothing,
+            "listpartitions":     self._fetch_list,
+            "newpartition":       self._fetch_nothing,
             # Audio Output Commands
             "disableoutput":      self._fetch_nothing,
             "enableoutput":       self._fetch_nothing,
             "toggleoutput":       self._fetch_nothing,
             "outputs":            self._fetch_outputs,
+            "outputset":          self._fetch_nothing,
             # Reflection Commands
             "config":             self._fetch_object,
             "commands":           self._fetch_list,
             "notcommands":        self._fetch_list,
-            "tagtypes":           self._fetch_list,
             "urlhandlers":        self._fetch_list,
             "decoders":           self._fetch_plugins,
             # Client to Client
@@ -317,13 +343,13 @@ class MPDClient:
             raise IteratingError("Cannot execute '%s' while iterating" %
                                  command)
         if self._pending:
-            raise PendingCommandError("Cannot execute '%s' with "
-                                      "pending commands" % command)
+            raise PendingCommandError(
+                "Cannot execute '%s' with pending commands" % command)
         retval = self._commands[command]
         if self._command_list is not None:
             if not callable(retval):
-                raise CommandListError("'%s' not allowed in command list" %
-                                        command)
+                raise CommandListError(
+                    "'%s' not allowed in command list" % command)
             self._write_command(command, args)
             self._command_list.append(retval)
         else:
@@ -478,6 +504,17 @@ class MPDClient:
     def _fetch_neighbors(self):
         return self._fetch_objects(["neighbor"])
 
+    def _fetch_composite(self):
+        obj = {}
+        for key, value in self._read_pairs():
+            key = key.lower()
+            obj[key] = value
+            if key == 'binary':
+                break
+        by = self._read_line()
+        obj['data'] = by.encode(errors='surrogateescape')
+        return obj
+
     @iterator_wrapper
     def _fetch_command_list(self):
         return self._read_command_list()
@@ -502,8 +539,8 @@ class MPDClient:
 
     def _connect_unix(self, path):
         if not hasattr(socket, "AF_UNIX"):
-            raise ConnectionError("Unix domain sockets not supported "
-                                  "on this platform")
+            raise ConnectionError(
+                "Unix domain sockets not supported on this platform")
         sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
         sock.connect(path)
         return sock
@@ -521,7 +558,9 @@ class MPDClient:
             sock = None
             try:
                 sock = socket.socket(af, socktype, proto)
+                sock.settimeout(CONNECTION_TIMEOUT)
                 sock.connect(sa)
+                sock.settimeout(None)
                 return sock
             except socket.error as socket_err:
                 err = socket_err
@@ -535,7 +574,8 @@ class MPDClient:
     def noidle(self):
         # noidle's special case
         if not self._pending or self._pending[0] != 'idle':
-            raise CommandError('cannot send noidle if send_idle was not called')
+            raise CommandError(
+                'cannot send noidle if send_idle was not called')
         del self._pending[0]
         self._write_command("noidle")
         return self._fetch_list()
@@ -569,7 +609,7 @@ class MPDClient:
             self._sock = self._connect_unix(host)
         else:
             self._sock = self._connect_tcp(host, port)
-        self._rfile = self._sock.makefile("r", encoding='utf-8')
+        self._rfile = self._sock.makefile("r", encoding='utf-8', errors='surrogateescape')
         self._wfile = self._sock.makefile("w", encoding='utf-8')
         try:
             self._hello()
